package com.sap.sapdb.DBMTest;
/*!
  -----------------------------------------------------------------------------
  module: EventDelete.java
  -----------------------------------------------------------------------------


    ========== licence begin  GPL
    Copyright (c) 2002-2005 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*/

/*
  -----------------------------------------------------------------------------
  includes
  -----------------------------------------------------------------------------
 */
import java.sql.*;
import com.sap.sapdb.testframe.driver.*;
import java.util.*;

/**
 * DBMServer
 * <xmp>
 * -----------------------------------------------------------------------------
 * Class:        EventDelete
 * -----------------------------------------------------------------------------
 * responsible:  ChristianH
 * special area: DBMServer Java Test
 * description:  DBMServer event_delete command tests
 * version:      7.4.*
 * copyright:    (c) 2002-2004 SAP AG
 * -----------------------------------------------------------------------------
 * </xmp>
 */

public class EventDelete extends BackupState
{

  public EventDelete() throws SQLException, TestDatabaseException
  {
    super();
  }
  /**
   * run() contains the test implementation
   */
  public void run()
  {
    DBMUtility oDBM           = new  DBMUtility(getDatabase(), this);
    boolean m_bEventFound     = false;

    //  create one global try-catch block to handle exceptions
    try
    {
      /**
       * event_delete --> BACKUP_PAGES
       */
      //Starting util session
      oDBM.execCmd("util_connect", DBMUtility.OK_OK);
      //With util session
      oDBM.execCmd("event_set BACKUP_PAGES 1000", DBMUtility.OK_OK);

      //Checking whether event is shown in event list
      oDBM.execCmd("event_list", DBMUtility.OK_OK, false);

      StringTokenizer stParam = new StringTokenizer(oDBM.getAnswer(),"\n");

      // Searching for syntax -->
      // BACKUP_PAGES            ON      LOW     1000            Next <n> pages moved
      while (stParam.hasMoreTokens()) {

        String m_sLine = stParam.nextToken().toString().toUpperCase();

        if (m_sLine.indexOf("BACKUP_PAGES") >= 0) {

          if (m_sLine.indexOf("ON") >= 0 &&
              m_sLine.indexOf("LOW") >= 0 &&
              m_sLine.indexOf("1000") >= 0 &&
              m_sLine.indexOf("Next <n> pages moved".toUpperCase()) >= 0) {

            m_bEventFound = true;
            break;
          }
        }
      }

      if (m_bEventFound == true) {

        addMessage("'event_list'", TestProtocol.INFO, "Creating event\n" + oDBM.getAnswer());
      } else {
        addMessage("'event_list'", TestProtocol.ERROR, "Creating event\n" + oDBM.getAnswer());
      }

      //Deleting event
      oDBM.execCmd("event_delete", DBMUtility.ERR_TOFEWPARAM);
      oDBM.execCmd("event_delete BACKUP_PAGES", DBMUtility.ERR_PARAM);
      oDBM.execCmd("event_delete BACKUP_PAGES 1000", DBMUtility.OK_OK);

      //Checking whether event is shown in event list
      oDBM.execCmd("event_list", DBMUtility.OK_OK, false);

      m_bEventFound = false;

      stParam = new StringTokenizer(oDBM.getAnswer(),"\n");

      // Searching for syntax -->
      // BACKUP_PAGES            OFF     NIL     0               Next <n> pages moved
      while (stParam.hasMoreTokens()) {

        String m_sLine = stParam.nextToken().toString().toUpperCase();

        if (m_sLine.indexOf("BACKUP_PAGES") >= 0) {

          if (m_sLine.indexOf("OFF") >= 0 &&
              m_sLine.indexOf("NIL") >= 0 &&
              m_sLine.indexOf("0") >= 0 &&
              m_sLine.indexOf("Next <n> pages moved".toUpperCase()) >= 0) {

            m_bEventFound = true;
            break;
          }
        }
      }

      if (m_bEventFound == true) {

        addMessage("'event_list'", TestProtocol.INFO, "Deleting event\n" + oDBM.getAnswer());
      } else {
        addMessage("'event_list'", TestProtocol.ERROR, "Deleting event\n" + oDBM.getAnswer());
      }

      //Ending util session
      oDBM.execCmd("util_release", DBMUtility.OK_OK);
    }
    // catch any kind of exception and write it to the protocol
    catch (Exception e) {
      // standard error handler

      this.handleExceptions(e);
    } // end try catch

    // clean up, i.e. close statements and connections
    finally {
      try {
        // print all parameters from the parameter object to the protocol.
        printParametersToProtocol();
      }
      catch (Exception e) {
        // do nothing
      } // end try catch
    } // end finally
  } // end run
  /**
   * Returns (defines) the name of the test class. If this method doesn't exist, the test driver
   * will set the name to the name of the java class. To make the test name independend
   * from the java implementation, this method should return a unique string.
   * Attention: The uniqueness is not automatically checked by anybody.
   * @return The UNIQUE name of the test class
   */
  public static String getTestClassId() {
    return "EventDelete";
  } // end getTestClassId
} //end of class MediumGet